@@ -147,12 +147,9 @@ module java.base {
         java.management,
         java.rmi,
         openj9.sharedclasses;
-    exports com.ibm.tools.attach.target to
-        jdk.attach,
-        jdk.jcmd,
-        jdk.management;
     exports com.sun.security.ntlm to java.security.sasl;
     exports jdk.internal to jdk.jfr;
+    exports jdk.internal.event to jdk.jfr;
     exports jdk.internal.jimage to jdk.jlink;
     exports jdk.internal.jimage.decompressor to jdk.jlink;
     exports jdk.internal.jmod to
@@ -234,10 +231,13 @@ module java.base {
     exports jdk.internal.vm.annotation to
         jdk.internal.vm.ci,
         jdk.unsupported;
-    exports openj9.management.internal to java.management;
-    exports openj9.tools.attach.diagnostics.base to
+    exports openj9.internal.tools.attach.target to
         jdk.attach,
-        jdk.jcmd;
+        jdk.jcmd,
+        jdk.management;
+    exports openj9.management.internal to
+        java.management,
+        openj9.jvm;
     exports sun.net to
         java.net.http,
         jdk.naming.dns;
